@@ -16,6 +16,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+FILE_LICENCE ( GPL2_OR_LATER );
+
 #include <string.h>
 #include <pxe.h>
 #include <realmode.h>
@@ -173,12 +175,11 @@ static int undinet_call ( struct undi_nic *undinic, unsigned int function,
 	__asm__ __volatile__ ( REAL_CODE ( "pushw %%es\n\t"
 					   "pushw %%di\n\t"
 					   "pushw %%bx\n\t"
-					   "lcall *%c3\n\t"
+					   "lcall *undinet_entry_point\n\t"
 					   "addw $6, %%sp\n\t" )
 			       : "=a" ( exit ), "=b" ( discard_b ),
 			         "=D" ( discard_D )
-			       : "p" ( __from_data16 ( &undinet_entry_point )),
-			         "b" ( function ),
+			       : "b" ( function ),
 			         "D" ( __from_data16 ( &undinet_params ) )
 			       : "ecx", "edx", "esi", "ebp" );
 
@@ -482,8 +483,7 @@ static void undinet_poll ( struct net_device *netdev ) {
 					 undi_isr.Frame.offset, frag_len );
 			if ( iob_len ( iobuf ) == len ) {
 				/* Whole packet received; deliver it */
-				netdev_rx ( netdev, iobuf );
-				iobuf = NULL;
+				netdev_rx ( netdev, iob_disown ( iobuf ) );
 				/* Etherboot 5.4 fails to return all packets
 				 * under mild load; pretend it retriggered.
 				 */
@@ -670,22 +670,26 @@ int undinet_probe ( struct undi_device *undi ) {
 	undi->flags |= UNDI_FL_STARTED;
 
 	/* Bring up UNDI stack */
-	memset ( &undi_startup, 0, sizeof ( undi_startup ) );
-	if ( ( rc = undinet_call ( undinic, PXENV_UNDI_STARTUP, &undi_startup,
-				   sizeof ( undi_startup ) ) ) != 0 )
-		goto err_undi_startup;
-	memset ( &undi_initialize, 0, sizeof ( undi_initialize ) );
-	if ( ( rc = undinet_call ( undinic, PXENV_UNDI_INITIALIZE,
-				   &undi_initialize,
-				   sizeof ( undi_initialize ) ) ) != 0 )
-		goto err_undi_initialize;
+	if ( ! ( undi->flags & UNDI_FL_INITIALIZED ) ) {
+		memset ( &undi_startup, 0, sizeof ( undi_startup ) );
+		if ( ( rc = undinet_call ( undinic, PXENV_UNDI_STARTUP,
+					   &undi_startup,
+					   sizeof ( undi_startup ) ) ) != 0 )
+			goto err_undi_startup;
+		memset ( &undi_initialize, 0, sizeof ( undi_initialize ) );
+		if ( ( rc = undinet_call ( undinic, PXENV_UNDI_INITIALIZE,
+					   &undi_initialize,
+					   sizeof ( undi_initialize ))) != 0 )
+			goto err_undi_initialize;
+	}
+	undi->flags |= UNDI_FL_INITIALIZED;
 
 	/* Get device information */
 	memset ( &undi_info, 0, sizeof ( undi_info ) );
 	if ( ( rc = undinet_call ( undinic, PXENV_UNDI_GET_INFORMATION,
 				   &undi_info, sizeof ( undi_info ) ) ) != 0 )
 		goto err_undi_get_information;
-	memcpy ( netdev->ll_addr, undi_info.PermNodeAddress, ETH_ALEN );
+	memcpy ( netdev->hw_addr, undi_info.PermNodeAddress, ETH_ALEN );
 	undinic->irq = undi_info.IntNumber;
 	if ( undinic->irq > IRQ_MAX ) {
 		DBGC ( undinic, "UNDINIC %p invalid IRQ %d\n",
@@ -693,7 +697,7 @@ int undinet_probe ( struct undi_device *undi ) {
 		goto err_bad_irq;
 	}
 	DBGC ( undinic, "UNDINIC %p is %s on IRQ %d\n",
-	       undinic, eth_ntoa ( netdev->ll_addr ), undinic->irq );
+	       undinic, eth_ntoa ( netdev->hw_addr ), undinic->irq );
 
 	/* Get interface information */
 	memset ( &undi_iface, 0, sizeof ( undi_iface ) );
@@ -701,8 +705,9 @@ int undinet_probe ( struct undi_device *undi ) {
 				   &undi_iface,
 				   sizeof ( undi_iface ) ) ) != 0 )
 		goto err_undi_get_iface_info;
-	DBGC ( undinic, "UNDINIC %p has type %s and link speed %d\n",
-	       undinic, undi_iface.IfaceType, undi_iface.LinkSpeed );
+	DBGC ( undinic, "UNDINIC %p has type %s, speed %d, flags %08x\n",
+	       undinic, undi_iface.IfaceType, undi_iface.LinkSpeed,
+	       undi_iface.ServiceFlags );
 	if ( strncmp ( ( ( char * ) undi_iface.IfaceType ), "Etherboot",
 		       sizeof ( undi_iface.IfaceType ) ) == 0 ) {
 		DBGC ( undinic, "UNDINIC %p Etherboot 5.4 workaround enabled\n",
@@ -732,11 +737,13 @@ int undinet_probe ( struct undi_device *undi ) {
 	memset ( &undi_cleanup, 0, sizeof ( undi_cleanup ) );
 	undinet_call ( undinic, PXENV_UNDI_CLEANUP, &undi_cleanup,
 		       sizeof ( undi_cleanup ) );
+	undi->flags &= ~UNDI_FL_INITIALIZED;
  err_undi_startup:
 	/* Unhook UNDI stack */
 	memset ( &stop_undi, 0, sizeof ( stop_undi ) );
 	undinet_call ( undinic, PXENV_STOP_UNDI, &stop_undi,
 		       sizeof ( stop_undi ) );
+	undi->flags &= ~UNDI_FL_STARTED;
  err_start_undi:
 	netdev_nullify ( netdev );
 	netdev_put ( netdev );
@@ -759,19 +766,26 @@ void undinet_remove ( struct undi_device *undi ) {
 	/* Unregister net device */
 	unregister_netdev ( netdev );
 
-	/* Shut down UNDI stack */
-	memset ( &undi_shutdown, 0, sizeof ( undi_shutdown ) );
-	undinet_call ( undinic, PXENV_UNDI_SHUTDOWN, &undi_shutdown,
-		       sizeof ( undi_shutdown ) );
-	memset ( &undi_cleanup, 0, sizeof ( undi_cleanup ) );
-	undinet_call ( undinic, PXENV_UNDI_CLEANUP, &undi_cleanup,
-		       sizeof ( undi_cleanup ) );
+	/* If we are preparing for an OS boot, or if we cannot exit
+	 * via the PXE stack, then shut down the PXE stack.
+	 */
+	if ( ! ( undi->flags & UNDI_FL_KEEP_ALL ) ) {
 
-	/* Unhook UNDI stack */
-	memset ( &stop_undi, 0, sizeof ( stop_undi ) );
-	undinet_call ( undinic, PXENV_STOP_UNDI, &stop_undi,
-		       sizeof ( stop_undi ) );
-	undi->flags &= ~UNDI_FL_STARTED;
+		/* Shut down UNDI stack */
+		memset ( &undi_shutdown, 0, sizeof ( undi_shutdown ) );
+		undinet_call ( undinic, PXENV_UNDI_SHUTDOWN, &undi_shutdown,
+			       sizeof ( undi_shutdown ) );
+		memset ( &undi_cleanup, 0, sizeof ( undi_cleanup ) );
+		undinet_call ( undinic, PXENV_UNDI_CLEANUP, &undi_cleanup,
+			       sizeof ( undi_cleanup ) );
+		undi->flags &= ~UNDI_FL_INITIALIZED;
+
+		/* Unhook UNDI stack */
+		memset ( &stop_undi, 0, sizeof ( stop_undi ) );
+		undinet_call ( undinic, PXENV_STOP_UNDI, &stop_undi,
+			       sizeof ( stop_undi ) );
+		undi->flags &= ~UNDI_FL_STARTED;
+	}
 
 	/* Clear entry point */
 	memset ( &undinet_entry_point, 0, sizeof ( undinet_entry_point ) );
